\name{khan}
\alias{khan}
\title{Khan microarray data}
\description{
 The \code{khan} data frame has 2309 rows and 65 columns.
These are one of the datasets data used in the Tibshirani et al paper in PNAS
on nearest shrunken centroids.
}
\format{
}
\details{
The first row contains the sample labels. The first two columns of gene ids and names. 
The remaining values of the matrix are gene expression values.
}
\source{
  }
\references{
}
\keyword{datasets}


\eof
\name{pamr.adaptthresh}
\alias{pamr.adaptthresh}
\title{ A function  to adaptive choose threshold scales, for
  use in pamr.train}
\description{A function  to adaptive choose threshold scales, for
  use in pamr.train}

\usage{
pamr.adaptthresh(object, ntries = 10, reduction.factor = 0.9, full.out = FALSE)
}

\arguments{
  
  \item{object}{The result of a call to pamr.train }
  \item{ntries}{Number of iterations to use in algorithm}
  \item{reduction.factor}{Amount by which a scaling is reduced in one
    step of the algorithm}
\item{full.out}{Should full output be returned? Default FALSE}
}    


\details{
  \code{pamr.adaptthresh} Adaptively searches for  set of
  good threshold scales.  The baseline (default) scale is 1 for
  each class. The idea is that for easy to classify classes,
  the threshold scale can be increased without increasing the error
  rate for that class, and resulting in fewer genes needed for the
  classification rule. The scalings from pamr.adaptthresh are then used
  in pamr.train, and pamr.cv. The results may be better than those obtained
  with the default values of threshold.scale.
}


\references{
  
  Robert Tibshirani, Trevor Hastie, Balasubramanian Narasimhan, and Gilbert Chu. "Diagnosis of multiple cancer types by shrunken centroids of gene expression"  PNAS 2002 99:6567-6572 (May 14). 
  
  Robert Tibshirani,  Trevor Hastie, Balasubramanian Narasimhan, and Gilbert Chu (2002).
  Class prediction by  
  nearest shrunken centroids,with applications
  to DNA microarrays. Stanford tech report.
}




\author{ Trevor Hastie, Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }

\examples{
set.seed(120)
x <- matrix(rnorm(1000*20),ncol=20)
y <- sample(c(1:4),size=20,replace=TRUE)
mydata <- list(x=x,y=y)
mytrain <-   pamr.train(mydata)
new.scales <- pamr.adaptthresh(mytrain)

 
mytrain2 <- pamr.train(mydata, threshold.scale=new.scales)

myresults2 <- pamr.cv(mytrain2, mydata)

}
\keyword{ }


\eof
\name{pamr.batchadjust}
\alias{pamr.batchadjust}
\title{ A function to mean-adjust microarray data by batches}
\description{
  A function to mean-adjust microarray data by batches
}
\usage{
pamr.batchadjust(data)
}

\arguments{
  \item{data}{The input data. A list with components: x- an expression
    genes in the rows, samples in the columns, and y-  a vector of
    the class labels for each sample, and batchlabels- a vector of batch
    labels for each sample.}
  This object if the same form as that produced by pamr.from.excel.}


\details{
  \code{pamr.batchadjust} 
  does a genewise one-way ANOVA adjustment for expression values.
  Let x(i,j) be the expression for gene i in sample j.
  Suppose sample j in in batch b, and let B be the set of all samples
  in batch b. Then \code{pamr.batchadjust}  adjusts  x(i,j) to
  x(i,j) - mean[x(i,j)]  where the mean is taken over all samples j in B
}

\value{ A data object  of the same form as the input data, with x
  replaced by the adjusted x}

\references{}



\author{ Trevor Hastie,Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }

\examples{
set.seed(120)
#generate some data
x <- matrix(rnorm(1000*20),ncol=20)
y <- sample(c(1:4),size=20,replace=TRUE)
batchlabels <- sample(c(1:5),size=20,replace=TRUE)
mydata <- list(x=x,y=factor(y),batchlabels=factor(batchlabels))

mydata2 <- pamr.batchadjust(mydata)
}

\keyword{ }


\eof
\name{pamr.confusion}
\alias{pamr.confusion}
\title{ A function giving a table of true versus predicted values,
  from a nearest shrunken centroid fit.}
\description{ A function giving a table of true versus predicted values,
  from a nearest shrunken centroid fit.}

\usage{
pamr.confusion(fit, threshold, extra=TRUE)
}

\arguments{
  
  \item{fit}{The result of a call to pamr.train or pamr.cv}
  \item{threshold}{The desired threshold value}
\item{extra}{Should the  classwise and overall error rates be returned?
Default TRUE}
  
  
}    

\details{
  \code{pamr.confusion} Gives a cross-tabulation of true versus
  predicted classes for the fit returned by pamr.train or pamr.cv,
  at the specified threshold.
}


\references{}




\author{ Trevor Hastie, Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }

\examples{
set.seed(120)
x <- matrix(rnorm(1000*20),ncol=20)
y <- sample(c(1:4),size=20,replace=TRUE)
mydata <- list(x=x,y=y)
mytrain <-   pamr.train(mydata)
mycv <- pamr.cv(mytrain,mydata)
pamr.confusion(mytrain,  threshold=2)
pamr.confusion(mycv,  threshold=2)
 
}
\keyword{ }


\eof
\name{pamr.confusion.survival}
\alias{pamr.confusion.survival}
\title{ Compute confusin matrix from pamr survival fit}
\description{computes confusion matrix for (survival.time,censoring) outcome
 based on fit object "fit" and class predictions "yhat"
 soft response probabilities for (survival.time,censoring) are first estimated
  using Kaplan-Meier method applied to training data}


\usage{
pamr.confusion.survival(fit, survival.time, censoring.status, yhat)
}

\arguments{
  
  \item{fit}{The result of a call to pamr.train or pamr.cv}
   \item{survival.time}{Survival time}
\item{censoring.status}{censoring status}
\item{yhat}{class predictions}
  
}    

\details{}
}


\references{}




\author{ Trevor Hastie, Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }

\examples{
}
\keyword{ }


\eof
\name{pamr.cv}
\alias{pamr.cv}
\title{ A function to cross-validate the nearest shrunken centroid
  classifier}
\description{A function to cross-validate the nearest shrunken centroid
  classifier produced by pamr.train}

\usage{
pamr.cv(fit, data,  nfold = NULL, folds = NULL,...)
}

\arguments{
  \item{fit}{The result of a call to pamr.train}
  \item{data}{A list with at least two components: x- an expression
    genes in the rows, samples in the columns), and y-  a vector of
    the class labels for each sample. Same form as data object
    used by pamr.train.}
  \item{nfold}{Number of cross-validation folds. Default is the
    smallest
    class size}
  \item{folds}{A list with nfold components, each component a vector of
    indices of the samples in that fold. By default a (random) balanced
    cross-validation is used}.
  \item{...} {Any additional arguments that are to be passed to pamr.train}
}    
\details{
  \code{pamr.cv} carries out cross-validation for a nearest shrunken
  centroid classifier.
}
\value{
  A list with components
  \item{threshold} {A vector of the thresholds tried in the shrinkage}
  \item{errors}{The number of cross-validation  errors for each threshold value}
  \item{loglik}{The cross-validated multinomial log-likelihood value
    for each threshold value}
  \item{size}{A vector of the number of genes that survived the
    thresholding, for each threshold value tried.}.
  \item{yhat}{A matrix of size n by nthreshold, containing the
    cross-validated
    class predictions for each threshold value,  in each column}
  \item{prob}{A matrix of size n by nthreshold, containing the
    cross-validated
    class probabilities for each threshold value,  in each column}
  \item{folds}{ The cross-validation folds used}
  \item{call}{The calling sequence used}
  
}

\references{}


\author{ Trevor Hastie,Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }

\examples{
set.seed(120)
x <- matrix(rnorm(1000*20),ncol=20)
y <- sample(c(1:4),size=20,replace=TRUE)

mydata <- list(x=x,y=factor(y), geneid=as.character(1:nrow(x)), genenames=paste("g",as.character(1:nrow(x)),sep=""),)

mytrain <-   pamr.train(mydata)
mycv <- pamr.cv(mytrain,mydata)
}

\keyword{ }


\eof
\name{pamr.from.excel}
\alias{pamr.from.excel}
\title{ A function to read in a text file saved from Excel}
\description{ A function to read in a text file saved from Excel.
  The spreadsheet is assumed to be of the format used by the SAM program.}

\usage{
pamr.from.excel(file, ncols, sample.labels = FALSE, batch.labels = FALSE) 
}

\arguments{
  \item{file}{Character name of a text file. This is assumed to be a tab-delimited text file saved from an excel spreadsheet from  "SAM". 
    The spreadsheet has one row of expression values per gene. In addition
    there is one information row and two information columns.  The first row
    has class labels for each of the samples. The first column had gene identifiers,
    and the second column has gene names. In the SAM program, for the multiclass
    option, the samples must be labelled 1,2,3 etc. Here we allow general labels,
    like "lymphoma", "colon cancer" etc 
  }
  \item{ncols}{Number of columns in file}
  \item{sample.labels}{Optional argument. If true, "file" is assumed to
    have an additional row at the top, consisting of two blank cells
    followed by a sample labels for each of the columns. If available, these sample labels
    are used by various plotting routines.} 
  \item{batch.labels}{Optional argument. If true, "file" is assumed to
    have an additional row at the top, consisting of two blank cells
    followed by a batch labels for each of the columns.
    If sample.labels=T as well, the row of batch labels are assumed to 
    come after the row of sample labels.
    The batch labels are used by the function pamr.batchadjust.
  } 
}  
\details{
  \code{pamr.from.excel} {Reads in the  text file "file", and creates an
    object with components x (the matrix of expression values), y- a vector of
    class labels for each sample, geneid- a vector of gene identifiers and
    genenames- a vector of gene names}
}
\value{
  A list with components
  \item{x} {the matrix of expression values}
  \item{y} {a vector of
    class labels for each sample,} 
  \item{geneid} {a vector of gene identifiers}
  \item{genenames}{a vector of gene names}
  \item{samplelabels}{a vector of sample labels, if provided in "file"}
  \item{batchlabels}{a vector of batch labels, if provided in "file"}
}

\references{ }




\author{ Trevor Hastie, Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }

\keyword{ }


\eof
\name{pamr.geneplot}
\alias{pamr.geneplot}
\title{ A function to  plot the genes that surive the thresholding
  from the nearest shrunken centroid classifier}
\description{ A function to plot the genes that survive the thresholding,
  from the nearest shrunken centroid classifier produced by pamr.train}

\usage{
pamr.geneplot(fit, data, threshold)
}

\arguments{
  \item{fit}{The result of a call to pamr.train}
  \item{data}{The input data.  In the same format as the input data for
    pamr.train}
  
  \item{threshold}{The desired threshold value}
}    

\details{
  \code{pamr.geneplot} Plots the raw gene expression for genes
  that survive the specified threshold. Plot is stratified by class. 
  Plot is set up to display only up to about 20 or 25 genes,
  otherwise it gets too crowded. Hence threshold should be chosen to
  yield at most about 20 or 25 genes.
}


\references{}




\author{ Trevor Hastie, Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }

\examples{
set.seed(120)
x <- matrix(rnorm(1000*20),ncol=20)
y <- sample(c(1:4),size=20,replace=TRUE)
mydata <- list(x=x,y=y)
mytrain <-   pamr.train(mydata)
pamr.geneplot(mytrain, mydata, threshold=1.6)
 
}
\keyword{ }


\eof
\name{pamr-internal}
\title{Internal pamr functions}
\alias{pamr.pairscore}
\alias{pamr.pvalue.survival}
\alias{pamr.score.to.class1}
\alias{pamr.score.to.class2}
\alias{pamr.knnimpute.old}
\alias{pamr.cube.root}
\alias{print.nsc}
\alias{print.nsccv}
\description{Internal pamr functions}
\usage{
pamr.pairscore(x, pair.ind=NULL)
pamr.pvalue.survival(group, survival.time, censoring.status, ngroup.survival)
pamr.score.to.class1(x, scores, cutoff=2, n.class=2)
pamr.score.to.class2(x, scores, cutoff=2, n.pc=1, n.class=2)
pamr.knnimpute.old(data, k = 10) 
pamr.cube.root(x)
print.nsc(x, ...)
print.nsccv(x, ...)
}
\author{Balasubramanian Narasimhan
 and Rob Tibshirani}
\details{
  These are not to be called by the user.
}
\keyword{internal}


\eof
\name{pamr.knnimpute}
\alias{pamr.knnimpute}
\title{A function to impute missing expression data}
\description{A function to impute missing expression data, using
  nearest neighbor averaging.}


\usage{
pamr.knnimpute(data ,k = 10, rowmax = 0.5, colmax = 0.8, maxp = 1500)
}

\arguments{
  \item{data}{The PAM input data. A list with components: \code{x}, an expression
    matrix with 
    genes in the rows, samples in the columns, and \code{y}, a vector of
    the class labels for each sample. Same form as used by pamr.train,
    and same as that produced by \code{pamr.from.excel}}
  \item{k}{Number of neighbors to be used in the
    imputation (default=10)}
  \item{rowmax}{The maximum percent missing data allowed in any row
    (default 50\%). For any rows with more than \code{rowmax}\% missing
    are imputed using the overall mean per sample.}
  \item{colmax}{The maximum percent missing data allowed in any column
    (default 80\%). If any column has more than \code{colmax}\% missing data,
    the program halts and reports an error.}
  \item{maxp}{The largest block of genes imputed using the knn
    algorithm inside \code{pamr.knnimpute} (default
    1500); larger blocks are divided by two-means clustering
    (recursively) prior to imputation. If \code{maxp=p}, only knn
    imputation is done.}
} 
  
\details{
  \code{pamr.knnimpute} 
  uses k-nearest neighbors in the space of genes to impute missing
  expression values.
  
  For each gene with missing values, we find the k nearest neighbors using
  a Euclidean metric, confined to the columns for which that gene is NOT
  missing. Each candidate neighbor might be missing some of the
  coordinates used to calculate the distance. In this case we average the
  distance from the non-missing coordinates. Having found the k nearest
  neighbors for a gene, we impute the missing elements by averaging those
  (non-missing) elements of its neighbors. This can fail if ALL the
  neighbors are missing in a particular element. In this case we use the
  overall column mean for that block of genes.
  
  Since nearest neighbor imputation costs
  \eqn{O(p\log(p))}{O(p*log(p))} operations per gene, where p is the
  number of rows, the computational time can be excessive for large p and
  a large number of missing rows. Our strategy is to break blocks with
  more than \code{maxp} genes into two smaller blocks using two-mean
  clustering. This is done recursively till all blocks have less than
  \code{maxp} genes. For each block, knn imputation is done separately.
  We have set the default value of \code{maxp} to 1500. Depending on the
  speed of the machine, and number of samples, this number might be
  increased. Making it too small is counter-productive, because the
  number of two-mean clustering algorithms will increase.
}

\value{
  \item{data} {The input data list, with x replaced by the 
    imputed version of x}
}

\references{
  Hastie, T., Tibshirani, R., Sherlock, G., Eisen, M., Brown, P. and
  Botstein, D., Imputing Missing Data for Gene Expression Arrays,
  Stanford University Statistics Department Technical report (1999), http://www-stat.stanford.edu/~hastie/Papers/missing.pdf
  
  Olga Troyanskaya, Michael Cantor, Gavin Sherlock, Pat Brown, Trevor Hastie, Robert Tibshirani, David Botstein and Russ B. Altman, Missing value estimation methods for DNA microarrays BIOINFORMATICS Vol. 17 no. 6, 2001 Pages 520-525 
}

\author{ Trevor Hastie, Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu}

\keyword{}


\eof
\name{pamr.listgenes}
\alias{pamr.listgenes}
\title{ A function to list the genes that survive the thresholding,
  from the nearest shrunken centroid classifier}
\description{ A function to list the genes that survive the thresholding,
  from the nearest shrunken centroid classifier produced by pamr.train}

}
\usage{
pamr.listgenes(fit, data,  threshold, genenames=FALSE)
}

\arguments{
  \item{fit}{The result of a call to pamr.train}
  \item{data}{The input data.  In the same format as the input data for
    pamr.train}
  
  \item{threshold}{The desired threshold value}
  \item{genenames}{Include genenames in the list? If yes, they
    are taken from "data". Default is false (geneid is always included
    in the list).}
}    

\details{
  \code{pamr.listgenes} List the geneids, and standardized centroids for
  each class, for genes surviving at the given threshold.
}


\references{}

\author{ Trevor Hastie, Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }

\examples{

#generate some data
set.seed(120)
x <- matrix(rnorm(1000*20),ncol=20)
y <- sample(c(1:4),size=20,replace=TRUE)

mydata <- list(x=x,y=factor(y), geneid=as.character(1:nrow(x)), genenames=paste("g",as.character(1:nrow(x)),sep=""),)


#train classifier
mytrain<-   pamr.train(mydata)

pamr.listgenes(mytrain, mydata, threshold=1.6)
 
}
\keyword{ }


\eof
\name{pamr.makeclasses}
\alias{pamr.makeclasses}
\title{A function to interactively define classes from a clustering tree}
\description{function to interactively define classes from a clustering tree}
\usage{
pamr.makeclasses(data,sort.by.class=FALSE,...)
}

\arguments{
  \item{data}{The input data. A list with components: x- an expression
    genes in the rows, samples in the columns, and y-  a vector of
    the class labels for each sample, and batchlabels- a vector of batch
    labels for each sample.
    This object if the same form as that produced by pamr.from.excel.}
  \item{sort.by.class}{Optional argument. If true, the clustering tree
    is forced to put all samples in the same class  (as defined by the
    class labels y in `data') together in the tree. This is useful
    if a regrouping of classes is desired. Eg: given classes 1,2,3,4
    you want to define new classes (1,3) vs (2,4)  or 2 vs (1,3)}
  \item{...} {Any additional arguments to be passed to hclust}
}

\details{
  \code{pamr.makeclasses} 
  Using this function the user interactively defines a new set of classes,
  to be used in pamr.train, pamr.cv etc. 
  After invoking pamr.makeclasses, a clustering tree is drawn.
  This callss the R function \code{hclust}, and any arguments for
  \code{hclust} can be passed to it.
  Using the left button, the user clicks at the junction point defining
  the subgroup 1. More groups can be added to class 1 by clicking
  on further junction points. The user ends the definition of class 1 by
  clicking on the rightmost button [in Windows,  an additional menu appears
  and he chooses Stop] . This process is continued for classes 2,3 etc.
  Note that some sample may be left out of the new classes.
  Two consecutive clicks of the right button ends the definition for all classes.
  
  At the end, the clustering is redrawn, with the new class labels shown.
  
  Note: this function is "fragile". The user must click close to the junction
  point, to avoid confusion with other junction points. Classes 1,2,3..
  cannot have samples in common (if they do, an Error message will appear).
  If the function is confused about the desired choices, it will
  complain and ask the user to rerun pamr.makeclasses. The user should
  also check that the labels on the  final redrawn cluster tree agrees with the desired 
  classes.
}

\value{A vector of class labels 1,2,3...
  If a component is NA (missing), then the
  sample is not assigned to any class.
  This vector should be assigned to the newy component of data, for use in pamr.train etc.
  Note that pamr.train uses the class labels in the component ``newy'' if
  it is present. Otherwise it uses the data labels ``y''.
}

\references{}



\author{ Trevor Hastie, Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }

\examples{
set.seed(120)
#generate some data
x <- matrix(rnorm(1000*40),ncol=40)
y <- sample(c(1:4),size=40,replace=TRUE)
batchlabels <- sample(c(1:5),size=40,replace=TRUE)

mydata <- list(x=x,y=factor(y),batchlabels=factor(batchlabels), geneid=as.character(1:nrow(x)), genenames=paste("g",as.character(1:nrow(x)),sep=""),)

# mydata$newy <- pamr.makeclasses(mydata)    Run this and define some new classes

train <- pamr.train(mydata)
}

\keyword{ }


\eof
\name{pamr.menu}
\alias{pamr.menu}
\title{A function that interactively leads the user through a PAM analysis}
\description{A function that interactively leads the user through a PAM analysis}


\usage{
pamr.menu(data)
}

\arguments{
  \item{data}{A list with at least two components: x- an expression
    genes in the rows, samples in the columns), and y-  a vector of
    the class labels for each sample. Same form as data object
    used by pamr.train.}
}    

\details{
  \code{pamr.menu} provides a menu for training, cross-validating
  and plotting a nearest shrunken centroid analysis.
}
\value{
}

\references{}


\author{ Trevor Hastie, Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }

\examples{
set.seed(120)
x <- matrix(rnorm(1000*20),ncol=20)
y <- sample(c(1:4),size=20,replace=TRUE)
mydata <- list(x=x,y=y)
#  pamr.menu(mydata)
}

\keyword{ }


\eof
\name{pamr.plotcen}
\alias{pamr.plotcen}
\title{ A function to plot the shrunken class centroids, from the nearest shrunken centroid classifier}
\description{ A function to plot the shrunken class centroids, from the
  nearest shrunken centroid classifier produced by pamr.train}

}
\usage{
pamr.plotcen(fit, data,  threshold)
}

\arguments{
  \item{fit}{The result of a call to pamr.train}
  \item{data}{The input data, in the same form as that used
    by pamr.train},
  \item{threshold}{The desired threshold value}
}    

\details{
  \code{pamr.plotcen} plots the shrunken class centroids for each
  class,
  for genes surviving the threshold for at least once class. If
  genenames
  are included in "data", they are added to the plot. Note: for many
  classes
  and long gene names, this plot may need some manual prettying.
}


\references{
}




\author{ Trevor Hastie, Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }

\examples{
set.seed(120)
x <- matrix(rnorm(1000*20),ncol=20)
y <- sample(c(1:4),size=20,replace=TRUE)
mydata <- list(x=x,y=y,genenames=as.character(1:1000))
mytrain <-   pamr.train(mydata)
mycv <- pamr.cv(mytrain,mydata)
pamr.plotcen(mytrain, mydata,threshold=1.6)
 
}
\keyword{ }


\eof
\name{pamr.plotcv}
\alias{pamr.plotcv}
\title{ A function to plot the cross-validated  error curves from
  the nearest shrunken centroid
  classifier} 
\description{A function to plot the cross-validated error curves
  the nearest shrunken centroid
  classifier}

\usage{
pamr.plotcv(fit)
}

\arguments{
  \item{fit}{The result of a call to pamr.cv}
}    

\details{
  \code{pamr.plotcv} plots the cross-validated misclassification error
  curves,   from nearest shrunken
  centroid classifier. An overall plot, and a plot by class, are produced.
}
\value{
}

\references{}


\author{ Trevor Hastie,Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }

\examples{
set.seed(120)
x <- matrix(rnorm(1000*20),ncol=20)
y <- sample(c(1:4),size=20,replace=TRUE)
mydata <- list(x=x,y=y)
mytrain <-   pamr.train(mydata)
mycv <-  pamr.cv(mytrain, mydata)
pamr.plotcv(mycv)
}

\keyword{ }


\eof
\name{pamr.plotcvprob}
\alias{pamr.plotcvprob}
\title{ A function to plot the cross-validated sample probabilities from
  the nearest shrunken centroid
  classifier} 
\description{A function to plot the cross-validated sample probabilities from 
  the nearest shrunken centroid
  classifier}

\usage{
pamr.plotcvprob(fit, data, threshold)
}

\arguments{
  \item{fit}{The result of a call to pamr.cv}
  \item{data}{A list with at least two components: x- an expression
    genes in the rows, samples in the columns), and y-  a vector of
    the class labels for each sample. Same form as data object
    used by pamr.train.}
  \item{threshold}{Threshold value to be used}
}    

\details{
  \code{pamr.plotcvprob} plots the cross-validated sample probabilities  the from nearest shrunken
  centroid classifier, stratified by the true classses.
}
\value{
}

\references{}


\author{ Trevor Hastie,Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }

\examples{
set.seed(120)
x <- matrix(rnorm(1000*20),ncol=20)
y <- sample(c(1:4),size=20,replace=TRUE)
mydata <- list(x=x,y=y)
mytrain <-   pamr.train(mydata)
mycv <-  pamr.cv(mytrain,mydata)
pamr.plotcvprob(mycv,mydata,threshold=1.6)



}

\keyword{ }


\eof
\name{pamr.plotstrata}
\alias{pamr.plotstrata}
\title{ A function to plot the survival curves in each Kaplan Meier stratum}
\description{A function to plot the survival curves in each Kaplan Meier stratum}


\usage{
pamr.plotstrata(fit, survival.time, censoring.status)
}

\arguments{
  \item{fit}{The result of a call to pamr.train}
  \item{survival.time}{Vector of survival times}
 \item{censoring.status}{Vector of censoring status values}
}    

\details{
}
\value{
}

\references{}


\author{ Trevor Hastie,Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }

\examples{

gendata<-function(n=100, p=2000){
  tim <- 3*abs(rnorm(n))
  u<-runif(n,min(tim),max(tim))
  y<-pmin(tim,u)
   ic<-1*(tim<u)
m <- median(tim)
x<-matrix(rnorm(p*n),ncol=n)
  x[1:100, tim>m] <-  x[1:100, tim>m]+3
  return(list(x=x,y=y,ic=ic))
}

# generate training data; 2000 genes, 100 samples

junk<-gendata(n=100)
y<-junk$y
ic<-junk$ic
x<-junk$x
d <- list(x=x,survival.time=y, censoring.status=ic,
geneid=as.character(1:nrow(x)), genenames=paste("g",as.character(1:nrow(x)),sep=
""),)

# train model
a3<- pamr.train(d, ngroup.survival=2)


pamr.plotstrata(a3, d$survival.time, d$censoring.status)
}

\keyword{ }


\eof
\name{pamr.plotsurvival}
\alias{pamr.plotsurvival}
\title{ A function to plots Kaplan-Meier curves stratified by a group variable}
\description{ A function to plots Kaplan-Meier curves stratified by a group variable}

\usage{
pamr.plotsurvival(group, survival.time, censoring.status)
}

\arguments{
  \item{group}{A grouping factor}
  \item{survival.time}{ Vector of survival times}
  \item{censoring.status}{Vector of censoring status values: 1=died, 0=censored }
}

\details{}
\value{}

\references{}


\author{ Trevor Hastie,Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }


\examples{

gendata<-function(n=100, p=2000){
  tim <- 3*abs(rnorm(n))
  u<-runif(n,min(tim),max(tim))
  y<-pmin(tim,u)
   ic<-1*(tim<u)
m <- median(tim)
x<-matrix(rnorm(p*n),ncol=n)
  x[1:100, tim>m] <-  x[1:100, tim>m]+3
  return(list(x=x,y=y,ic=ic))
}

# generate training data; 2000 genes, 100 samples

junk<-gendata(n=100)
y<-junk$y
ic<-junk$ic
x<-junk$x
d <- list(x=x,survival.time=y, censoring.status=ic,
geneid=as.character(1:nrow(x)), genenames=paste("g",as.character(1:nrow(x)),sep=
""),)

# train model
a3<- pamr.train(d, ngroup.survival=2)

#make class predictions

yhat <- pamr.predict(a3,d$x, threshold=1.0)

pamr.plotsurvival(yhat, d$survival.time, d$censoring.status)

}


\keyword{ }


\eof
\name{pamr.predict}
\alias{pamr.predict}
\title{ A function giving prediction information,
  from a nearest shrunken centroid fit.}
\description{A function giving prediction information,
  from a nearest shrunken centroid fit}

}
\usage{
pamr.predict(fit, newx, threshold, type= c("class", "posterior", "centroid", "nonzero"), prior = fit$prior, threshold.scale = fit$threshold.scale)
}

\arguments{
  
  \item{fit}{The result of a call to pamr.train }
\item{newx}{Matrix of features at which predictions are to be made}
  \item{threshold}{The desired threshold value}
  \item{type}{Type of prediction desired: class predictions, posterior
    probabilities, (unshrunken) class centroids, vector of  genes
    surviving the threshold}
  \item{prior}{Prior probabilities for each class. Default is that
    specified in "fit"}
  \item{threshold.scale}{Additional scaling factors to be applied
    to the thresholds. Vector of length equal to the number of
    classes.
    Default is that
    specified in "fit".}
}


\details{
  \code{pamr.predict} Give a cross-tabulation of true versus
  predicted classes for the fit returned by pamr.train or pamr.cv,
  at the specified threshold
}


\references{ }




\author{ Trevor Hastie, Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }

\examples{
set.seed(120)
x <- matrix(rnorm(1000*20),ncol=20)
y <- sample(c(1:4),size=20,replace=TRUE)
mydata <- list(x=x,y=y)
mytrain <-   pamr.train(mydata)
mycv <- pamr.cv(mytrain,mydata)
pamr.predict(mytrain, mydata$x , threshold=1)
 
}
\keyword{ }


\eof
\name{pamr.predictmany}
\alias{pamr.predictmany}
\title{ A function giving prediction information for many threshold values,
  from a nearest shrunken centroid fit.}
\description{A function giving prediction information for many threshold values,
  from a nearest shrunken centroid fit}

}
\usage{
pamr.predictmany(fit, newx, threshold=fit$threshold, prior =fit$prior, threshold.scale = fit$
        threshold.scale, ...)
}

\arguments{
  \item{fit}{The result of a call to pamr.train }
\item{newx}{Matrix of features at which predictions are to be made}
  \item{threshold}{The desired threshold values}
  \item{prior}{Prior probabilities for each class. Default is that
    specified in "fit"}
  \item{threshold.scale}{Additional scaling factors to be applied
    to the thresholds. Vector of length equal to the number of
    classes.
    Default is that
    specified in "fit".}
\item{...}{Additional arguments to be passed to pamr.predict}
}


\details{
}


\references{ }




\author{ Trevor Hastie, Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }

\examples{
set.seed(120)
x <- matrix(rnorm(1000*20),ncol=20)
y <- sample(c(1:4),size=20,replace=TRUE)
mydata <- list(x=x,y=y)
mytrain <-   pamr.train(mydata)

pamr.predictmany(mytrain, mydata$x)
 
}
\keyword{ }


\eof
\name{pamr.surv.to.class2}
\alias{pamr.surv.to.class2}
\title{A function to  assign observations to categories, based on their
survival times.}
\description{A function to  assign observations to categories, based on their
survival times.}


\usage{
pamr.surv.to.class2(y, icens, cutoffs=NULL, n.class=NULL,  class.names=NULL, newy=y, newic=icens)
}

\arguments{
  \item{y}{vector of survival times}
\item{icens}{Vector of censorng status values: 1=died, 0=censored}
\item{cutoffs}{Survival time cutoffs for categories. Default NULL}
\item{n.class}{Number of classes to create: if cutoffs is NULL,
     n.class equal classes are created.}
\item{class.names}{Character names for classes}
\item{newy}{New set of survival times, for which probabilities are computed
(see below). Default is y}
\item{newic}{New set of censoring statuses, for which probabilities are computed
(see below). Default is icens}

}
  
\details{
\code{pamr.pamr.surv.to.class2}  splits observations into categories
based on their survival times and the Kaplan-Meier estimates.
For example if n.class=2, it makes
two categories, one below the median survival, the other above.
For each observation (newy, ic), it then computes the probability
of that observation falling in each category. For an uncensored observation
that probability is just  1 or 0 depending on when the death occurred.
For a censored observation, the probabilities are based on the Kaplan
Meier and are typically between 0 and 1.
}

\value{
  \item{class} {The category labels}
 \item{prob}{The estimates class probabilities}
 \item{cutoffs}{The cutoffs used}
}

\author{ Trevor Hastie, Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu}

\examples{

gendata<-function(n=100, p=2000){
  tim <- 3*abs(rnorm(n))
  u<-runif(n,min(tim),max(tim))
  y<-pmin(tim,u)
   ic<-1*(tim<u)
m <- median(tim)
x<-matrix(rnorm(p*n),ncol=n)
  x[1:100, tim>m] <-  x[1:100, tim>m]+3
  return(list(x=x,y=y,ic=ic))
}

# generate training data; 2000 genes, 100 samples

junk<-gendata(n=100)
y<-junk$y
ic<-junk$ic
x<-junk$x
d <- list(x=x,survival.time=y, censoring.status=ic,
geneid=as.character(1:nrow(x)), genenames=paste("g",as.character(1:nrow(x)),sep=
""),)

# train model
a3<- pamr.train(d, ngroup.survival=2)

# generate test data
junkk<- gendata(n=500)

dd <- list(x=junkk$x, survival.time=junkk$y, censoring.status=junkk$ic)

# compute soft labels
proby <-  pamr.surv.to.class2(dd$survival.time, dd$censoring.status,
             n.class=a3$ngroup.survival)$prob

}
\keyword{}


\eof
\name{pamr.test.errors.surv.compute}
\alias{pamr.test.errors.surv.compute}
\title{ A function giving a table of true versus predicted values,
  from a nearest shrunken centroid fit from survival data.}
\description{ A function giving a table of true versus predicted values,
  from a nearest shrunken centroid fit from survival data.}

\usage{
pamr.test.errors.surv.compute(proby, yhat)
}

\arguments{
  
  \item{proby}{Survival class probabilities, from pamr.surv.to.class2}
  \item{yhat}{Estimated class labels, from pamr.predict}
  
  
}    

\details{
  \code{pamr.test.errors.surv.compute} computes the erros between the
true 'soft" class labels proby and the estimated ones "yhat"
}


\references{}




\author{ Trevor Hastie, Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }

\examples{
 
gendata<-function(n=100, p=2000){
  tim <- 3*abs(rnorm(n))
  u<-runif(n,min(tim),max(tim))
  y<-pmin(tim,u)
   ic<-1*(tim<u)
m <- median(tim)
x<-matrix(rnorm(p*n),ncol=n)
  x[1:100, tim>m] <-  x[1:100, tim>m]+3
  return(list(x=x,y=y,ic=ic))
}

# generate training data; 2000 genes, 100 samples

junk<-gendata(n=100)
y<-junk$y
ic<-junk$ic
x<-junk$x
d <- list(x=x,survival.time=y, censoring.status=ic, 
geneid=as.character(1:nrow(x)), genenames=paste("g",as.character(1:nrow(x)),sep=
""),)

# train model
a3<- pamr.train(d, ngroup.survival=2)

# generate test data
junkk<- gendata(n=500)

dd <- list(x=junkk$x, survival.time=junkk$y, censoring.status=junkk$ic)

# compute soft labels
proby <-  pamr.surv.to.class2(dd$survival.time, dd$censoring.status,
             n.class=a3$ngroup.survival)$prob


# make class predictions for test data
yhat <- pamr.predict(a3,dd$x, threshold=1.0)

# compute test errors

pamr.test.errors.surv.compute(proby, yhat)

}
\keyword{ }


\eof
\name{pamr.to.excel}
\alias{pamr.to.excel}
\title{ A function to write out a data object into a tab-delimited text file}
\description{ A function to write out a data object into a tab-delimited text file}


\usage{
pamr.to.excel(data,file,trace=TRUE)
}

\arguments{
  \item{data}{A data object, of the same form as is read in by pamr.from.excel.
    Must have components x (the matrix of expression values), y- a vector of
    class labels for each sample, geneid- a vector of gene identifiers and
    genenames- a vector of gene names. Optional components: samplelabels and
    batchlabels, both character vectors}
\item{file}{Character name of a text file.}
\item{trace}{Optional argument. If true, progress in writing out file is reported.}
}
\details{
  \code{pamr.to.excel} {
    writes out the data object into a tab-delimited text file, of the same
    form as is read in by pamr.from.excel.
    Useful for writing out data that has been imputed by pamr.knnimpute or
    adjusted by pamr.batchadjust. Note- this function writes the file out
    one line at a time, and hence can take a while for big datasets.
  }
}




\author{ Trevor Hastie, Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }

\keyword{ }


\eof
\name{pamr.train}
\alias{pamr.train}
\title{ A function to train a nearest shrunken centroid
  classifier}
\description{
  A function that computes a nearest shrunken centroid for gene
  expression
  (microarray) data
}
\usage{
pamr.train(data, gene.subset=NULL, sample.subset=NULL,
         threshold = NULL, n.threshold = 30, 
        scale.sd = TRUE, threshold.scale = NULL, se.scale = NULL, offset.percent = 50,
         hetero=NULL, prior = NULL, remove.zeros = TRUE, sign.contrast="both",
ngroup.survival = 2)
}

\arguments{
  \item{data}{The input data. A list with components: x- an expression
    genes in the rows, samples in the columns), and y-  a vector of
    the class labels for each sample. 
    Optional components-
    genenames,
    a vector of gene names, and geneid- a vector of gene identifiers.}
  \item{gene.subset}{Subset of genes to be used.  Can be either
    a logical vector of length total number of genes, or a list
    of integers of the row numbers of the genes to be used}
  \item{sample.subset} {Subset of samples to be used.  Can be either
    a logical vector of length total number of samples, or a list
    of integers of the column numbers of the samples to be used.}
  \item{threshold}{A vector of threshold values for the centroid
    shrinkage.
    Default is a set of 30 values chosen by the software}
  \item{n.threshold}{Number of threshold values desired (default 30)}
  \item{scale.sd}{Scale each threshold by the wthin class standard
    deviations? Default: true}
  \item{threshold.scale}{Additional scaling factors to be applied
    to the thresholds. Vector of length equal to the number of
    classes.
    Default- a vectors of ones.}
  \item{se.scale}{Vector of scaling factors for the within class
    standard errors. Default is sqrt(1/n.class-1/n), where n is
    the overall sample size and n.class is the sample sizes in each
    class. This default adjusts for different class sizes.}
  \item{offset.percent}{Fudge factor added to  the denominator of each t-statistic,
    expressed as a percentile of the gene standard deviation values.
    This is a small positive quantity to penalize genes with
    expression
    values near zero, which can result in very large ratios. This
    factor
    is expecially impotant for Affy data. Default
    is the median of the standard deviations of each gene.}
 \item{hetero}{Should a heterogeneity transformation be done?
    If yes, hetero must be set to one of the class labels (see Details below).
    Default is no (hetero=NULL) }
  \item{prior}{Vector of length the number of classes, representing
    prior probabilities for each of the classes. The prior is used
    in Bayes rule for making class prediction. Default is NULL, and prior
is then taken to be  n.class/n,
    where  n is the overall sample size and n.class is the sample
    sizes in   each class.}
\item{remove.zeros}{Remove threshold values yielding zero genes? Default TRUE}
  \item{sign.contrast}{Directions of allowed deviations of class-wise average gene  expression from the 
    overall average  gene expression. Default is ``both'' (positive or negative).
    Can also  be set to ``positive'' or ``negative''.}
\item{ngroup.survival}{Number of groups formed for survival data. Default 2}
}
\details{
  \code{pamr.train} fits a nearest shrunken centroid classifier to gene
  expression data. Details may be found in the PNAS paper referenced
  below. One feature not described there is "heterogeneity analysis".
  Suppose there are two classes labelled "A" and "B".
  CLass "A" is considered a normal class, and "B" an abnormal class.
  Setting hetero="A" transforms   expression values x[i,j] to
  |x[i,j]- mean(x[i,j])| where the mean is taken only over samples in
  class "A". The transformed feature values are then used in Pam.
  This is useful when the abnormal class "B" is heterogeneous, i.e.
  a given gene might have higher expresion than normal for some
  class "B" samples, and lower for others.
  With more than 2 classes, each class is centered on the class specified
  by hetero.
}

\value{
  A list with components
  \item{y}{The outcome classes.} 
  \item{yhat}{A matrix of predicted classes, each column representing
    the results from one threshold.}.
  \item{prob}{A array of predicted class probabilities. of dimension
    n by nclass by n.threshold. n is the number samples, nclass is the number
    of classes, n.threshold is the number of thresholds tried}
  \item{centroids}{A matrix of (unshrunken) class centroids, n by
    nclass}
  \item{hetero}{Value of hetero used in call to pamr.train}
  \item{norm.cent}{Centroid of "normal" group, if hetero was specified}
  \item{centroid.overall} {A vector containing the (unshrunken) overall
    centroid (all classes together)}
  \item{sd}{A vector of the standard deviations for each gene}
  \item{threshold} {A vector of the threshold tried in the shrinkage}
  \item{nonzero}{A vector of the number of genes that survived the
    thresholding, for each threshold value tried}
  \item{threshold.scale}{A vector of threshold scale factors that were
    used}
  \item{se.scale}{A vector of standard error scale factors that were
    used}
  \item{call}{The calling sequence used}
  \item{prior}{The prior probabilities used}
  \item{errors}{The number of trainin errors for each threshold value}
}

\references{
  Robert Tibshirani, Trevor Hastie, Balasubramanian Narasimhan, and Gilbert Chu 
  Diagnosis of multiple cancer types by shrunken centroids of gene expression 
  PNAS 99: 6567-6572.   Available at www.pnas.org}




\author{ Trevor Hastie,Robert Tibshirani, Balasubramanian Narasimhan, and Gilbert Chu  }

\examples{
#generate some data
set.seed(120)
x <- matrix(rnorm(1000*20),ncol=20)
y <- sample(c(1:4),size=20,replace=TRUE)
mydata <- list(x=x,y=factor(y))

#train classifier
results<-   pamr.train(mydata)

# train classifier on all  data except class 4
results2 <- pamr.train(mydata,sample.subset=(mydata$y!=4))
 
# train classifier on  only the first 500 genes
results3 <- pamr.train(mydata,gene.subset=1:500)

}
\keyword{ }


\eof
